@@ -338,14 +338,14 @@
 
   # Variance of HT estimator under SRS
   if (is.null(period)) {
-           var_srs_HT <- var_srs(Y2a, w = w_design)
+           var_srs_HT <- var_srs(Y2a, w = w_design)$varsrs
        } else {
            period_agg <- unique(period)
            lin1 <- lapply(1:nrow(period_agg), function(i) {
                           per <- period_agg[i,][rep(1, nrow(Y2a)),]
                           ind <- (rowSums(per == period) == ncol(period))
                           data.table(period_agg[i,], 
-                                     var_srs(Y2a[ind], w = w_design[ind]))
+                                     var_srs(Y2a[ind], w = w_design[ind])$varsrs)
                         })
            var_srs_HT <- rbindlist(lin1)
       }
@@ -355,14 +355,14 @@
 
   # Variance of calibrated estimator under SRS
   if (is.null(period)) {
-           var_srs_ca <- var_srs(Y3, w = w_final)
+           var_srs_ca <- var_srs(Y3, w = w_final)$varsrs
       } else {
            period_agg <- unique(period)
            lin1 <- lapply(1:nrow(period_agg), function(i) {
                           per <- period_agg[i,][rep(1, nrow(Y2a)),]
                           ind <- (rowSums(per == period) == ncol(period))
                           data.table(period_agg[i,], 
-                                     var_srs(Y3[ind], w = w_final[ind]))
+                                     var_srs(Y3[ind], w = w_final[ind])$varsrs)
                         })
            var_srs_ca <- rbindlist(lin1)
         }
@@ -374,7 +374,7 @@
   # Total estimation
   Y_nov <- Z_nov <- .SD <- NULL
 
-  hY <- data.table(Y1*w_final)
+  hY <- data.table(Y1 * w_final)
   if (is.null(period)) { Y_nov <- hY[, lapply(.SD, sum, na.rm=TRUE), .SDcols = names(Y1)]
                 } else { hY <- data.table(period, hY)
                          Y_nov <- hY[, lapply(.SD, sum, na.rm=TRUE), keyby=names(period), .SDcols = names(Y1)]
